@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 1999, 2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1999 - 2004 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -85,7 +86,7 @@ manual_cancel_pending_events(Service_t * service_p)
 	timer_cancel(manual->timer);
     }
     if (manual->arp) {
-	arp_cancel_probe(manual->arp);
+	arp_client_cancel_probe(manual->arp);
     }
     return;
 }
@@ -128,10 +129,10 @@ manual_start(Service_t * service_p, IFEventID_t evid, void * event_data)
 	      break;
 	  }
 	  manual_cancel_pending_events(service_p);
-	  arp_probe(manual->arp, 
-		    (arp_result_func_t *)manual_start, service_p,
-		    (void *)IFEventID_arp_e, G_ip_zeroes,
-		    manual->our_ip);
+	  arp_client_probe(manual->arp, 
+			   (arp_result_func_t *)manual_start, service_p,
+			   (void *)IFEventID_arp_e, G_ip_zeroes,
+			   manual->our_ip);
 	  break;
       }
       case IFEventID_arp_e: {
