@@ -4,19 +4,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -31,11 +32,13 @@
 #include <stdlib.h>
 #include <sys/errno.h>
 #include <sys/types.h>
+#include <sys/param.h>
 #include <sys/stat.h>
 #include <sys/fcntl.h>
 #include <string.h>
 #include <sys/syslimits.h>
 #include <dirent.h>
+#include <syslog.h>
 
 #include "dynarray.h"
 #include "nbimages.h"
@@ -55,6 +58,27 @@ struct NBImageList_s {
 extern void
 my_log(int priority, const char *message, ...);
 
+static int
+cfstring_to_cstring_and_length_ext(CFStringRef cfstr, char * str, int len,
+				   boolean_t is_external)
+{
+    CFIndex		ret_len = 0;
+
+    CFStringGetBytes(cfstr, CFRangeMake(0, CFStringGetLength(cfstr)),
+		     kCFStringEncodingUTF8, '?', is_external,
+		     str, len - 1, &ret_len);
+    if (str != NULL) {
+	str[ret_len] = '\0';
+    }
+    return (ret_len + 1); /* leave 1 byte for nul-termination */
+}
+
+static __inline__ int
+cfstring_to_cstring_and_length(CFStringRef cfstr, char * str, int len)
+{
+    return (cfstring_to_cstring_and_length_ext(cfstr, str, len, FALSE));
+}
+
 static Boolean
 myCFStringArrayToCStringArray(CFArrayRef arr, char * buffer, int * buffer_size,
 			      int * ret_count)
@@ -79,23 +103,23 @@ myCFStringArrayToCStringArray(CFArrayRef arr, char * buffer, int * buffer_size,
 	CFStringRef	str;
 
 	str = CFArrayGetValueAtIndex(arr, i);
+	if (isA_CFString(str) == NULL) {
+	    return (FALSE);
+	}
 	if (buffer != NULL) {
 	    len = *buffer_size - space;
 	    if (len < 0) {
 		return (FALSE);
 	    }
 	}
-	CFStringGetBytes(str, CFRangeMake(0, CFStringGetLength(str)),
-			 kCFStringEncodingASCII, '\0',
-			 FALSE, offset, len - 1, &len);
+	len = cfstring_to_cstring_and_length(str, offset, len);
 	if (buffer != NULL) {
 	    strlist[i] = offset;
-	    offset[len] = '\0';
-	    offset += len + 1;
+	    offset += len;
 	}
-	space += len + 1;
+	space += len;
     }
-    *buffer_size = space;
+    *buffer_size = roundup(space, sizeof(char *));
     *ret_count = count;
     return (TRUE);
 }
@@ -328,19 +352,23 @@ NBImageList_free(NBImageListRef * l)
 }
 
 static boolean_t
-stat_file(char * dir, char * file)
+stat_file(const char * dir, const char * file)
 {
     char		path[PATH_MAX];
     struct stat		sb;
 
     snprintf(path, sizeof(path), "%s/%s", dir, file);
     if (stat(path, &sb) < 0) {
+#if 0
 	fprintf(stderr, "stat %s failed, %s\n",
 		path, strerror(errno));
+#endif 0
 	return (FALSE);
     }
     if ((sb.st_mode & S_IFREG) == 0) {
+#if 0
 	fprintf(stderr, "%s is not a file\n", path);
+#endif 0
 	return (FALSE);
     }
     return (TRUE);
@@ -377,9 +405,11 @@ dump_strlist(const char * * strlist, int count)
 static void
 NBImageEntry_print(NBImageEntryRef entry)
 {
-    printf("%-12s %-35.*s 0x%08x %-9s %-12s", 
+    int		i;
+
+    printf("%-12s %-35s 0x%08x %-9s %-12s", 
 	   entry->sharepoint.name,
-	   entry->name_length, entry->name,
+	   entry->name,
 	   entry->image_id, 
 	   NBImageTypeStr(entry->type),
 	   entry->bootfile);
@@ -401,6 +431,12 @@ NBImageEntry_print(NBImageEntryRef entry)
     default:
 	break;
     }
+    printf(" [");
+    for (i = 0; i < entry->archlist_count; i++) {
+	printf("%s%s", (i > 0) ? ", " : "", 
+	       entry->archlist[i]);
+    }
+    printf("]");
     if (entry->sysids != NULL) {
 	printf(" [ ");
 	dump_strlist(entry->sysids, entry->sysids_count);
@@ -439,18 +475,15 @@ static NBImageEntryRef
 NBImageEntry_create(NBSPEntryRef sharepoint, char * dir_name,
 		    char * dir_path, char * info_plist_path)
 {
+    CFArrayRef		archlist_prop;
+    int			archlist_space = 0;
     u_int16_t		attr = 0;
-    CFStringRef		bootfile;
+    CFStringRef		bootfile_prop;
+    int			bootfile_space = 0;
     boolean_t		diskless;
     NBImageEntryRef	entry = NULL;
-    char *		ent_bootfile = NULL;
-    char *		ent_name = NULL;
-    CFIndex		ent_name_len = 0;
-    char *		ent_private = NULL;
-    char *		ent_root_path = NULL;
-    char *		ent_shared = NULL;
-    char *		ent_tftp_path = NULL;
     boolean_t		filter_only;
+    int 		i;
     int32_t		idx_val = -1;
     CFNumberRef		idx;
     char *		image_file = NULL;
@@ -459,25 +492,28 @@ NBImageEntry_create(NBSPEntryRef sharepoint, char * dir_name,
     CFNumberRef		kind;
     int32_t		kind_val = -1;
     char *		mount_point = NULL;
-    CFStringRef		name;
+    CFStringRef		name_prop;
+    int			name_space;
     char *		offset;
-    char		path[PATH_MAX];
     CFPropertyListRef	plist;
-    CFStringRef		private;
-    CFStringRef		root_path;
+    CFStringRef		private_prop = NULL;
+    int			private_space = 0;
+    char		root_path[PATH_MAX];
+    CFStringRef		root_path_prop = NULL;
     struct in_addr	server_ip;
     char *              server_password;
     int                 server_port;
     char *              server_username;
-    CFStringRef		shared;
-    int			string_space = 0;
+    CFStringRef		shared_prop = NULL;
+    int			shared_space = 0;
+    int			tail_space = 0;
     int			sysids_space = 0;
     CFArrayRef		sysids_prop;
     char		tmp[PATH_MAX];
     CFStringRef		type;
     NBImageType		type_val = kNBImageTypeNone;
 
-    string_space = strlen(dir_name) + strlen(sharepoint->path) 
+    tail_space = strlen(dir_name) + strlen(sharepoint->path) 
 	+ strlen(sharepoint->name) + 3;
     plist = my_CFPropertyListCreateFromFile(info_plist_path);
     if (isA_CFDictionary(plist) == NULL) {
@@ -496,26 +532,17 @@ NBImageEntry_create(NBSPEntryRef sharepoint, char * dir_name,
 				   FALSE);
     filter_only = S_get_plist_boolean(plist, kNetBootImageInfoFilterOnly,
 				      FALSE);
-    name = CFDictionaryGetValue(plist, kNetBootImageInfoName);
-    if (isA_CFString(name) == NULL) {
+    name_prop = CFDictionaryGetValue(plist, kNetBootImageInfoName);
+    if (isA_CFString(name_prop) == NULL) {
 	fprintf(stderr, "missing/invalid Name property\n");
 	goto failed;
     }
-    ent_name_len = CFStringGetLength(name);
-    CFStringGetBytes(name, CFRangeMake(0, ent_name_len),
-		     kCFStringEncodingUTF8, '?', TRUE,
-		     tmp, BSDP_IMAGE_NAME_MAX,
-		     &ent_name_len);
-    if (ent_name_len == 0) {
-	printf("zero name length\n");
+    name_space = cfstring_to_cstring_and_length_ext(name_prop, NULL, 0, TRUE);
+    if (name_space <= 1) {
+	printf("empty Name property\n");
 	goto failed;
     }
-    ent_name = malloc(ent_name_len);
-    if (ent_name == NULL) {
-	goto failed;
-    }
-    bcopy(tmp, ent_name, ent_name_len);
-    string_space += ent_name_len;
+    tail_space += name_space;
 
     idx = CFDictionaryGetValue(plist, kNetBootImageInfoIndex);
     if (isA_CFNumber(idx) == NULL
@@ -577,32 +604,38 @@ NBImageEntry_create(NBSPEntryRef sharepoint, char * dir_name,
     }
     attr |= bsdp_image_attributes_from_kind(kind_val);
 
-    /* bootfile */
-    bootfile = CFDictionaryGetValue(plist, kNetBootImageInfoBootFile);
-    if (isA_CFString(bootfile) == NULL) {
-	fprintf(stderr, "missing/invalid BootFile property\n");
-	goto failed;
-    }
-    if (cfstring_to_cstring(bootfile, tmp, sizeof(tmp)) == FALSE) {
-	fprintf(stderr, "BootFile could not be converted\n");
-	goto failed;
-    }
-    if (stat_file(dir_path, tmp) == FALSE) {
-	fprintf(stderr, "BootFile does not exist\n");
-	goto failed;
-    }
-    ent_bootfile = strdup(tmp);
-    if (ent_bootfile == NULL) {
-	goto failed;
-    }
-    string_space += strlen(ent_bootfile) + 1;
+    /* architectures */
+    archlist_prop = CFDictionaryGetValue(plist, kNetBootImageInfoArchitectures);
+    if (archlist_prop != NULL) {
+	int archlist_count;
 
-    /* tftp_path */
-    snprintf(tmp, sizeof(tmp),
-	     NETBOOT_TFTP_DIRECTORY "/%s/%s/%s", sharepoint->name, 
-	     dir_name, ent_bootfile);
-    ent_tftp_path = strdup(tmp);
-    string_space += strlen(ent_tftp_path) + 1;
+	if (myCFStringArrayToCStringArray(archlist_prop, 
+					  NULL, &archlist_space,
+					  &archlist_count) == FALSE) {
+	    fprintf(stderr, 
+		    "Couldn't calculate Archlist length\n");
+	    goto failed;
+	}
+	if (archlist_count == 0) {
+	    fprintf(stderr, 
+		    "Empty ArchList array");
+	    goto failed;
+	}
+	tail_space += archlist_space;
+    }
+
+    /* bootfile */
+    bootfile_prop = CFDictionaryGetValue(plist, kNetBootImageInfoBootFile);
+    if (bootfile_prop != NULL && isA_CFString(bootfile_prop) == NULL) {
+	fprintf(stderr, "invalid BootFile property\n");
+	goto failed;
+    }
+    if (bootfile_prop == NULL) {
+	fprintf(stderr, "no BootFile property specified\n");
+	goto failed;
+    }
+    bootfile_space = cfstring_to_cstring_and_length(bootfile_prop, NULL, 0);
+    tail_space += bootfile_space;
 
     /* supported system ids */
     sysids_prop 
@@ -621,104 +654,104 @@ NBImageEntry_create(NBSPEntryRef sharepoint, char * dir_name,
 		    "Couldn't calculate EnabledSystemIdentifiers length\n");
 	    goto failed;
 	}
-	string_space += sysids_space;
+	if (sysids_count == 0) {
+	    /* if the list is empty, treat it as if it were not there at all */
+	    sysids_prop = NULL;
+	}
+	else {
+	    tail_space += sysids_space;
+	}
     }
     switch (type_val) {
     case kNBImageTypeClassic:
 	/* must have Shared */
-	shared = CFDictionaryGetValue(plist, kNetBootImageInfoSharedImage);
-	if (isA_CFString(shared) == NULL) {
+	shared_prop = CFDictionaryGetValue(plist, kNetBootImageInfoSharedImage);
+	if (isA_CFString(shared_prop) == NULL) {
 	    fprintf(stderr, "missing/invalid SharedImage property\n");
 	    goto failed;
 	}
-	if (cfstring_to_cstring(shared, tmp, sizeof(tmp)) == FALSE) {
-	    fprintf(stderr, "SharedImage could not be converted\n");
-	    goto failed;
-	}
+	shared_space = cfstring_to_cstring_and_length(shared_prop, tmp,
+						      sizeof(tmp));
 	if (stat_file(dir_path, tmp) == FALSE) {
 	    fprintf(stderr, "SharedImage does not exist\n");
 	    goto failed;
 	}
-	ent_shared = strdup(tmp);
-	if (ent_shared == NULL) {
-	    goto failed;
-	}
-	string_space += strlen(ent_shared) + 1;
+	tail_space += shared_space;
 
 	/* may have Private */
-	private 
+	private_prop 
 	    = isA_CFString(CFDictionaryGetValue(plist, 
 						kNetBootImageInfoPrivateImage));
-	if (private != NULL) {
-	    if (cfstring_to_cstring(private, tmp, sizeof(tmp))
-		&& stat_file(dir_path, tmp)) {
-		ent_private = strdup(tmp);
-		if (ent_private == NULL) {
-		    goto failed;
-		}
-		string_space += strlen(ent_private) + 1;
+	if (private_prop != NULL) {
+	    private_space = cfstring_to_cstring_and_length(private_prop, tmp,
+							   sizeof(tmp));
+	    if (stat_file(dir_path, tmp)) {
+		tail_space += private_space;
+	    }
+	    else {
+		private_prop = NULL;
 	    }
 	}
 	break;
     case kNBImageTypeNFS:
 	/* must have RootPath */
-	root_path = CFDictionaryGetValue(plist, kNetBootImageInfoRootPath);
-	if (isA_CFString(root_path) == NULL) {
+	root_path_prop = CFDictionaryGetValue(plist, kNetBootImageInfoRootPath);
+	if (isA_CFString(root_path_prop) == NULL) {
 	    fprintf(stderr, "missing/invalid RootPath property\n");
 	    goto failed;
 	}
-	if (cfstring_to_cstring(root_path, tmp, sizeof(tmp)) == FALSE) {
+	if (cfstring_to_cstring(root_path_prop, tmp, sizeof(tmp)) == FALSE) {
 	    fprintf(stderr, "RootPath could not be converted\n");
 	    goto failed;
 	}
 	if (stat_file(dir_path, tmp) == TRUE) {
-	    ent_root_path = strdup(tmp);
+	    strlcpy(root_path, tmp, sizeof(root_path));
 	}
 	else if (parse_nfs_path(tmp, &server_ip, &mount_point,
 				&image_file) == TRUE) {
 	    if (image_file) {
-		snprintf(path, sizeof(path), "nfs:%s:%s:%s",
+		snprintf(root_path, sizeof(root_path), "nfs:%s:%s:%s",
 			 inet_ntoa(server_ip), mount_point,
 			 image_file);
 	    }
 	    else {
-		snprintf(path, sizeof(path), "nfs:%s:%s",
+		snprintf(root_path, sizeof(root_path), "nfs:%s:%s",
 			 inet_ntoa(server_ip), mount_point);
 	    }
 	    indirect = TRUE;
-	    ent_root_path = strdup(path);
 	}
-	if (ent_root_path == NULL) {
+	else {
 	    goto failed;
 	}
-	string_space += strlen(ent_root_path) + 1;
+	tail_space += strlen(root_path) + 1;
 	break;
     case kNBImageTypeHTTP:
 	/* must have RootPath */
-	root_path = CFDictionaryGetValue(plist, kNetBootImageInfoRootPath);
-	if (isA_CFString(root_path) == NULL) {
+	root_path_prop = CFDictionaryGetValue(plist, kNetBootImageInfoRootPath);
+	if (isA_CFString(root_path_prop) == NULL) {
 	    fprintf(stderr, "missing/invalid RootPath property\n");
 	    goto failed;
 	}
-	if (cfstring_to_cstring(root_path, tmp, sizeof(tmp)) == FALSE) {
+	if (cfstring_to_cstring(root_path_prop, tmp, sizeof(tmp)) == FALSE) {
 	    fprintf(stderr, "RootPath could not be converted\n");
 	    goto failed;
 	}
 	if (stat_file(dir_path, tmp) == TRUE) {
-	    ent_root_path = strdup(tmp);
+	    strlcpy(root_path, tmp, sizeof(root_path));
 	}
 	else if (parse_http_path(tmp, &server_ip, &server_username,
 				 &server_password, &server_port,
 				 &image_file) == TRUE) {
 	    if (server_username && server_password) {
 		if (server_port != 0) {
-		    snprintf(path, sizeof(path), "http://%s:%s@%s:%d/%s",
+		    snprintf(root_path, sizeof(root_path),
+			     "http://%s:%s@%s:%d/%s",
 			     server_username, server_password, 
 			     inet_ntoa(server_ip), 
 			     server_port, image_file);
 		}
 		else {
-		    snprintf(path, sizeof(path), "http://%s:%s@%s/%s",
+		    snprintf(root_path, sizeof(root_path), "http://%s:%s@%s/%s",
 			     server_username, server_password, 
 			     inet_ntoa(server_ip),
 			     image_file);
@@ -726,29 +759,28 @@ NBImageEntry_create(NBSPEntryRef sharepoint, char * dir_name,
 	    } 
 	    else {
 		if (server_port != 0) {
-		    snprintf(path, sizeof(path), "http://%s:%d/%s",
+		    snprintf(root_path, sizeof(root_path), "http://%s:%d/%s",
 			     inet_ntoa(server_ip), server_port,
 			     image_file);
 		}
 		else {
-		    snprintf(path, sizeof(path), "http://%s/%s",
+		    snprintf(root_path, sizeof(root_path), "http://%s/%s",
 			     inet_ntoa(server_ip), image_file);
 		}
 	    }
 	    indirect = TRUE;
-	    ent_root_path = strdup(path);
 	}
-	if (ent_root_path == NULL) {
+	else {
 	    goto failed;
 	}
-	string_space += strlen(ent_root_path) + 1;
+	tail_space += strlen(root_path) + 1;
 	break;
     case kNBImageTypeBootFileOnly:
     default:
 	break;
     }
 
-    entry = (NBImageEntryRef)malloc(sizeof(*entry) + string_space);
+    entry = (NBImageEntryRef)malloc(sizeof(*entry) + tail_space);
     if (entry == NULL) {
 	goto failed;
     }
@@ -761,6 +793,57 @@ NBImageEntry_create(NBSPEntryRef sharepoint, char * dir_name,
 
     offset = (char *)(entry + 1);
 
+    /* do pointer arrays + strings first */
+
+    /* archlist */
+    if (archlist_prop != NULL) {
+	entry->archlist = (const char * *)offset;
+	(void)myCFStringArrayToCStringArray(archlist_prop, offset,
+					    &archlist_space,
+					    &entry->archlist_count);
+	offset += archlist_space;
+    }
+    else {
+	entry->arch = "ppc";
+	entry->archlist = &entry->arch;
+	entry->archlist_count = 1;
+    }
+
+    /* supported system ids */
+    if (sysids_prop != NULL) {
+	entry->sysids = (const char * *)offset;
+	(void)myCFStringArrayToCStringArray(sysids_prop, offset,
+					    &sysids_space,
+					    &entry->sysids_count);
+	qsort(entry->sysids, entry->sysids_count, sizeof(char *),
+	      my_ptrstrcmp);
+	offset += sysids_space;
+    }
+
+    /* ... then just strings */
+
+    /* bootfile */
+    entry->bootfile = offset;
+    (void)cfstring_to_cstring_and_length(bootfile_prop, offset,
+					 bootfile_space);
+    /* verify that bootfile exists for every defined architecture */
+    for (i = 0; i < entry->archlist_count; i++) {
+	char	path[PATH_MAX];
+	snprintf(path, sizeof(path), "%s/%s", entry->archlist[i],
+		 entry->bootfile);
+	if (stat_file(dir_path, path) == FALSE) {
+	    if (strcmp(entry->archlist[i], "ppc") == 0
+		&& stat_file(dir_path, entry->bootfile)) {
+		entry->ppc_bootfile_no_subdir = TRUE;
+	    }
+	    else {
+		fprintf(stderr, "BootFile does not exist\n");
+		goto failed;
+	    }
+	}
+    }
+    offset += bootfile_space;
+
     /* sharepoint */
     entry->sharepoint.path = offset;
     strcpy(entry->sharepoint.path, sharepoint->path);
@@ -776,75 +859,50 @@ NBImageEntry_create(NBSPEntryRef sharepoint, char * dir_name,
 
     /* name */
     entry->name = offset;
-    entry->name_length = ent_name_len;
-    bcopy(ent_name, entry->name, ent_name_len);
-    offset += ent_name_len; /* no nul termination */
-
-    /* bootfile */
-    entry->bootfile = offset;
-    strcpy(entry->bootfile, ent_bootfile);
-    offset += strlen(ent_bootfile) + 1;
-
-    /* tftp_path */
-    entry->tftp_path = offset;
-    strcpy(entry->tftp_path, ent_tftp_path);
-    offset += strlen(ent_tftp_path) + 1;
-
-    /* supported system ids */
-    if (sysids_space > 0) {
-	entry->sysids = (const char * *)offset;
-	(void)myCFStringArrayToCStringArray(sysids_prop, offset, &sysids_space,
-					    &entry->sysids_count);
-	qsort(entry->sysids, entry->sysids_count, sizeof(char *),
-	      my_ptrstrcmp);
-	offset += sysids_space;
-    }
+    (void)cfstring_to_cstring_and_length_ext(name_prop, offset, 
+					     name_space, TRUE);
+    entry->name_length = name_space - 1;
+    offset += name_space;
 
     switch (type_val) {
     case kNBImageTypeClassic:
 	entry->type_info.classic.shared = offset;
-	strcpy(entry->type_info.classic.shared, ent_shared);
-	offset += strlen(ent_shared) + 1;
-	if (ent_private != NULL) {
+	(void)cfstring_to_cstring_and_length(shared_prop, offset, shared_space);
+	offset += shared_space;
+	if (private_prop != NULL) {
 	    entry->type_info.classic.private = offset;
-	    strcpy(entry->type_info.classic.private, ent_private);
-	    offset += strlen(ent_private) + 1;
+	    (void)cfstring_to_cstring_and_length(private_prop, 
+						 offset, private_space);
+	    offset += private_space;
 	}
 	break;
     case kNBImageTypeNFS:
 	entry->type_info.nfs.root_path = offset;
-	strcpy(entry->type_info.nfs.root_path, ent_root_path);
-	offset += strlen(ent_root_path) + 1;
+	strcpy((char *)entry->type_info.nfs.root_path, root_path);
+	offset += strlen(root_path) + 1;
 	entry->type_info.nfs.indirect = indirect;
 	break;
     case kNBImageTypeHTTP:
 	entry->type_info.http.root_path = offset;
-	strcpy(entry->type_info.http.root_path, ent_root_path);
-	offset += strlen(ent_root_path) + 1;
+	strcpy((char *)entry->type_info.http.root_path, root_path);
+	offset += strlen(root_path) + 1;
 	entry->type_info.http.indirect = indirect;
 	break;
     default:
 	break;
     }
+#if 0
+    printf("tail_space %d - actual %d = %d\n",
+	   tail_space, (offset - (char *)(entry + 1)),
+	   tail_space - (offset - (char *)(entry + 1)));
+#endif 0
+    my_CFRelease(&plist);
+    return (entry);
 
  failed:
-    if (ent_name != NULL) {
-	free(ent_name);
-    }
-    if (ent_bootfile != NULL) {
-	free(ent_bootfile);
-    }
-    if (ent_tftp_path != NULL) {
-	free(ent_tftp_path);
-    }
-    if (ent_shared != NULL) {
-	free(ent_shared);
-    }
-    if (ent_private != NULL) {
-	free(ent_private);
-    }
-    if (ent_root_path != NULL) {
-	free(ent_root_path);
+    if (entry != NULL) {
+	free(entry);
+	entry = NULL;
     }
     my_CFRelease(&plist);
     return (entry);
@@ -852,8 +910,21 @@ NBImageEntry_create(NBSPEntryRef sharepoint, char * dir_name,
 
 boolean_t
 NBImageEntry_supported_sysid(NBImageEntryRef entry, 
+			     const char * arch,
 			     const char * sysid)
 {
+    boolean_t	found = FALSE;
+    int		i;
+
+    for (i = 0; i < entry->archlist_count; i++) {
+	if (strcmp(entry->archlist[i], arch) == 0) {
+	    found = TRUE;
+	    break;
+	}
+    }
+    if (found == FALSE) {
+	return (FALSE);
+    }
     if (entry->sysids == NULL) {
 	return (TRUE);
     }
@@ -975,7 +1046,8 @@ NBImageList_add_images(NBImageListRef image_list, NBSPEntryRef sharepoint)
 }
 
 NBImageEntryRef 
-NBImageList_default(NBImageListRef image_list, const char * sysid,
+NBImageList_default(NBImageListRef image_list, 
+		    const char * arch, const char * sysid,
 		    const u_int16_t * attrs, int n_attrs)
 {
     int			count;
@@ -986,7 +1058,7 @@ NBImageList_default(NBImageListRef image_list, const char * sysid,
     for (i = 0; i < count; i++) {
 	NBImageEntryRef	scan = dynarray_element(dlist, i);
 
-	if (NBImageEntry_supported_sysid(scan, sysid)
+	if (NBImageEntry_supported_sysid(scan, arch, sysid)
 	    && NBImageEntry_attributes_match(scan, attrs, n_attrs)) {
 	    return (scan);
 	}
@@ -1057,7 +1129,7 @@ NBImageList_print(NBImageListRef image_list)
 
 #ifdef TEST_NBIMAGES
 #if 0
-#include <syslog.h>
+#include <stdarg.h>
 void
 my_log(int priority, const char *message, ...)
 {
