@@ -6,19 +6,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -66,6 +67,9 @@
 #define NIPROP_NETBOOT_AFP_USER	"afp_user"
 #define NIPROP_NETBOOT_ARCH	"arch"
 #define NIPROP_NETBOOT_IMAGE_ID	"image_id"
+#define NIPROP_NETBOOT_IMAGE_INDEX "image_index"
+#define NIPROP_NETBOOT_IMAGE_KIND "image_kind"
+#define NIPROP_NETBOOT_IMAGE_IS_INSTALL "image_is_install"
 #define NIPROP_NETBOOT_NUMBER	"number"
 #define NIPROP_NETBOOT_SYSID	"sysid"
 #define NIPROP_NETBOOT_LAST_BOOT_TIME	"last_boot_time"
